package org.steveshipway.cheshirecat.cmd;

import java.util.ArrayList;
import java.util.List;

import org.steveshipway.cheshirecat.CheshireCat;
import org.steveshipway.cheshirecat.item.ItemWand;
import org.steveshipway.cheshirecat.lib.Constants;

import net.minecraft.command.ICommand;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.BlockPos;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.MathHelper;

public class WandCommand  implements ICommand {
		  private List aliases;
		  public WandCommand()
		  {
		    this.aliases = new ArrayList();
		    this.aliases.add("wand");
		  }

		  @Override
		  public String getName()
		  {
		    return "wand";
		  }

		  @Override
		  public String getCommandUsage(ICommandSender icommandsender)
		  {
		    return "wand <show|set <blueprint>>";
		  }

		  @Override
		  public List getAliases()
		  {
		    return this.aliases;
		  }

		  @Override
		  public void execute(ICommandSender icommandsender, String[] astring)
		  {
			  EntityPlayer player;
	          
	          if(icommandsender instanceof EntityPlayer){
	                  player = (EntityPlayer)icommandsender;
	          } else {
	                  icommandsender.addChatMessage(new ChatComponentText("Player Only Command"));
	                  return;
	          }
	          ItemStack item = player.getCurrentEquippedItem();
	          if (item == null) {
		    	  player.addChatMessage(new ChatComponentText("You are not holding anything"));
	        	  return;
	          }
	          if( !item.getUnlocalizedName().endsWith(Constants.MODID + "_" + ItemWand.name) ) {
		    	  player.addChatMessage(new ChatComponentText("That is not a wand you are holding"));
	        	  return;
	          }
		      if(astring.length < 1)
		      {
		    	  player.addChatMessage(new ChatComponentText("Usage: wand <show|set <blueprint>>"));
		    	  return;
		      }
		      NBTTagCompound nbt = item.getTagCompound();
		      if( nbt == null ) {
		    		nbt = new NBTTagCompound();
		    		nbt.setString("blueprint", "");
		    		item.setTagCompound(nbt);
		      }
	          if(astring[0].matches("show.*")) {
	        	  //String s = ((ItemWand)item.getItem()).getBlueprint();
	        	  String s = nbt.getString("blueprint");
	        	  if( s == null || s.matches("") ) {
			    	  player.addChatMessage(new ChatComponentText("Wand is not yet enchanted"));
	        	  } else {
			    	  player.addChatMessage(new ChatComponentText("Wand is set to build '"+s+"'"));
	        	  }
	        	  return;
	          }
	          if(!astring[0].matches("set.*")) {
		    	  player.addChatMessage(new ChatComponentText("Invalid subcommand '"+astring[0]+"'.  Should be 'set' or 'show'."));
	        	  return;
	          }

		      if(astring.length < 2)
		      {
		    	  player.addChatMessage(new ChatComponentText("No blueprint name given"));
		    	  return;
		      }
		      // now we can test for a valid blueprint
		      if( ! CheshireCat.portfolio.defines(astring[1]) ) {
		    	  player.addChatMessage(new ChatComponentText("Invalid blueprint name '" + astring[1] + "'"));
		    	  return;	    	  
		      }
		      nbt.setString("blueprint",astring[1]);
		      player.addChatMessage(new ChatComponentText("Wand enchanted to create '" + astring[1] + "'"));
		  }

		  @Override
		  public boolean canCommandSenderUse(ICommandSender icommandsender)
		  {
			  // is the player in create mode or holding the wand?
			  EntityPlayer player;
	          
	          if(icommandsender instanceof EntityPlayer){
	                  player = (EntityPlayer)icommandsender;
	          } else {
	                  icommandsender.addChatMessage(new ChatComponentText("Player Only Command"));
	                  return false;
	          }

	          ItemStack item = player.getCurrentEquippedItem();
	          if (item == null) {
	              icommandsender.addChatMessage(new ChatComponentText("You are not holding a wand, or indeed anything"));
				  return false;	        	  
	          }
	          if( item.getUnlocalizedName().endsWith(Constants.MODID + "_" + ItemWand.name) ) {
	        	  return true;
	          }
	          
              icommandsender.addChatMessage(new ChatComponentText("That is not a wand you are holding!"));
			  return false;
		  }

		  @Override
		  public List addTabCompletionOptions(ICommandSender icommandsender,String[] astring,BlockPos bp)
		  {
			  if( astring.length > 1 ) {
			  // should add the list of blueprints to them...
				  return CheshireCat.portfolio.list(astring[1]);
			  } else {
				  List l = new ArrayList();
				  l.add("set"); l.add("show");
				  return l;
			  }
		  }

		  @Override
		  public boolean isUsernameIndex(String[] astring, int i)
		  {
			  // none of the parameters are usernames
	   	      return false;
		  }

		  @Override
		  public int compareTo(Object o)
		  {
		      return 0;
		  }
}
