package org.steveshipway.cheshirecat.directive;

import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.steveshipway.cheshirecat.CheshireCat;
import org.steveshipway.cheshirecat.ChunkCoordinates;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.nbt.JsonToNBT;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
//import net.minecraft.util.ChunkCoordinates;
import net.minecraft.world.World;
import net.minecraft.world.chunk.Chunk;
//import net.minecraftforge.common.util.ForgeDirection;

public class AreaDirective extends Directive {
	private ChunkCoordinates locationa;
	private ChunkCoordinates locationb;
	private boolean soft = false;
	private IBlockState block = null;
	private String nbt = null; // possibly this should be a parsed NBT object
	private int dv = 0;

	public AreaDirective(String definition) {
    	Pattern p = Pattern.compile("(?i)^(?:\\s*area)?\\s*(~?)(\\w+:\\w+)(?:\\((\\d+)\\))?\\s+(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)\\s+(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)(\\s+\\{.*\\})?");
    	Matcher m = p.matcher(definition);
    	if( m.find() ) {
    		int dv = 0;
    		Block b = null;
    	
    		if( m.group(1).contains("~") ) {
    			this.soft = true;
    		}
    		b = Block.getBlockFromName(m.group(2));
    		if( b == null ) {
    			// invalid name
        		CheshireCat.logger.error("Invalid block name in AREA def: "+m.group(2));
    		} else {
        		if( m.group(3) != null ) {
        			dv = Integer.parseInt( m.group(3) );
        			// somehow, set the metadata on the block b
        		}
    			this.block = b.getBlockState().getBaseState();
    			this.block = this.block.getBlock().getStateFromMeta(dv);
    			this.dv = dv; // not really needed, but...
    			
    			// 2 is the block, 3 is the optional DV
	    		this.locationa = new ChunkCoordinates();
	    		this.locationa.posX = Integer.parseInt( m.group(4) );
	    		this.locationa.posY = Integer.parseInt( m.group(5) );
	    		this.locationa.posZ = Integer.parseInt( m.group(6) );
	
	    		this.locationb = new ChunkCoordinates();
	    		this.locationb.posX = Integer.parseInt( m.group(7) );
	    		this.locationb.posY = Integer.parseInt( m.group(8) );
	    		this.locationb.posZ = Integer.parseInt( m.group(9) );

    			this.nbt = m.group(10);
    		}
    	} else {
    		CheshireCat.logger.error("Cannot parse AREA definition: "+definition);
    	}
	}

	@Override
	public boolean build(World world,ChunkCoordinates location, int r) {
		ChunkCoordinates blockPosA = new ChunkCoordinates();
		ChunkCoordinates blockPosB = new ChunkCoordinates();
		boolean rv = true;
		if( block == null ) {
    		CheshireCat.logger.error("No valid block definition in AREA directive");
			return false;
		}
		if( world.isRemote ) {
    		CheshireCat.logger.error("Block placement called with remote world?  Should not happen.");
			return false;
		}
		// rotate offset by r
		blockPosA = rotateAndAdd(this.locationa,r,location);
		blockPosB = rotateAndAdd(this.locationb,r,location);
		if( blockPosA.posX > blockPosB.posX ) {
			int q = blockPosA.posX; blockPosA.posX = blockPosB.posX; blockPosB.posX = q;
		}
		if( blockPosA.posY > blockPosB.posY ) {
			int q = blockPosA.posY; blockPosA.posY = blockPosB.posY; blockPosB.posY = q;
		}
		if( blockPosA.posZ > blockPosB.posZ ) {
			int q = blockPosA.posZ; blockPosA.posZ = blockPosB.posZ; blockPosB.posZ = q;
		}
		// place the block(s), possibly with the NBT in a tileentity
		// iterate between the two locations
		BlockPos bp;
		Chunk chk;
		Block curBlock = null;
		IBlockState blockstate = null;
		IBlockState newstate = this.block;
		int curDv = 0;
		int ir = r;
		NBTTagCompound nbtTag = null;
		if( this.nbt != null ) {
			try {
				// read in nbtTag from this.nbt ... very version dependent
	//			nbtTag = (NBTTagCompound) JsonToNBT.func_150315_a(this.nbt); // forge 1.7.10
				nbtTag = (NBTTagCompound) JsonToNBT.func_180713_a(this.nbt); // forge 1.8
			} catch( Exception e ) {
				CheshireCat.logger.error("Unable to parse NBT '"+this.nbt+"': "+e.getMessage());		
				return false;
			}
		}
		newstate = CheshireCat.rotateState(newstate,r);
		for( int x = blockPosA.posX; x <= blockPosB.posX; x += 1 ) {
			for( int y = blockPosA.posY; y <= blockPosB.posY; y += 1 ) {
				for( int z = blockPosA.posZ; z <= blockPosB.posZ; z += 1 ) {
					bp = new BlockPos(x,y,z);
					blockstate = world.getBlockState(bp);
					curBlock = blockstate.getBlock();
					curDv = curBlock.getMetaFromState(blockstate);
					if(this.soft) {
						if( curBlock != Blocks.air && curBlock != Blocks.water && curBlock != Blocks.flowing_water ) {
							continue;  // soft block only placed in air or water
						}
					}
					
					if( curBlock != newstate.getBlock() || curDv != newstate.getBlock().getMetaFromState(newstate) ) {
						rv = world.setBlockState(bp,newstate,2);
					  	if( !rv ) {
					  		CheshireCat.logger.warn("Failed to place block in the world!");
					  		return false;
					  	}
					  	// if there was NBT specified, create tile entity
						if( nbtTag != null ) {
							nbtTag.setInteger("x", x);
							nbtTag.setInteger("y", y);
							nbtTag.setInteger("z", z);
							TileEntity te = TileEntity.createAndLoadEntity(nbtTag);
							world.setTileEntity(bp,te);
						}
					}
				}
				
			}
		}
		return true;
	}
	
	@Override
	public String asText() {
		String s = "AREA ";
		if( block == null) {
			return "# AREA undefined ";
		}
		if(soft) { 
			s += "~"; 
		}
		s += Block.blockRegistry.getNameForObject(block);
		s += " " + locationa.posX + "," + locationa.posY + "," + locationa.posZ ;
		s += " " + locationb.posX + "," + locationb.posY + "," + locationb.posZ ;
		if( nbt != null ) {
			s += " " + nbt;
		}
		return s;
	}
	
	
}
