package org.steveshipway.cheshirecat.directive;

import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.steveshipway.cheshirecat.CheshireCat;
import org.steveshipway.cheshirecat.ChunkCoordinates;

import net.minecraft.block.Block;
import net.minecraft.block.state.BlockState;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.nbt.JsonToNBT;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTUtil;
import net.minecraft.server.MinecraftServer;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
//import net.minecraft.util.ChunkCoordinates;
import net.minecraft.world.World;
//import net.minecraftforge.common.util.ForgeDirection;
import net.minecraft.world.chunk.Chunk;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyDirection;

public class BlockDirective extends Directive {
	// this is the main one!
	protected ChunkCoordinates location;
	protected boolean soft = false;
	protected IBlockState block = null;
	protected String nbt = null; // possibly this should be a parsed NBT object
	protected int dv = 0;        // no longer really needed since is included in IBlockState
	
	public BlockDirective(Block b,int dv, boolean soft,ChunkCoordinates offset, String nbt) {
		this.location = offset;
		this.block = b.getBlockState().getBaseState().getBlock().getStateFromMeta(dv);
		this.soft = soft;
		this.nbt = nbt;
		this.dv = dv;
	}
	public BlockDirective(String definition) {
    	Pattern p = Pattern.compile("(?i)^(?:\\s*block)?\\s*(~?)(\\w+:\\w+)(?:\\((\\d+)\\))?\\s+(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)(\\s+\\{.*\\})?");
    	Matcher m = p.matcher(definition);
    	if( m.find() ) {
    		Block b = null;
    		int dv = 0;
    		if( m.group(1).contains("~") ) {
    			this.soft = true;
    		}
    		// 2 is the block, 3 is the optional DV
    		// must identify the Block object from the m.group(2) text
    		b = Block.getBlockFromName(m.group(2));
    		if( b == null ) {
    			// invalid name
        		CheshireCat.logger.error("Invalid block name in BLOCK def: "+m.group(2));
    		} else {
    			if( m.group(3) != null ) {
        			dv = Integer.parseInt( m.group(3) );
        			// somehow, set the metadata on the block b
        		}
    			this.block = b.getBlockState().getBaseState();
    			this.block = this.block.getBlock().getStateFromMeta(dv);
    			this.dv = dv; // not really needed, but...
    			
    			// coordinates
    			this.location = new  ChunkCoordinates();
    			this.location.posX = Integer.parseInt( m.group(4) );
    			this.location.posY = Integer.parseInt( m.group(5) );
    			this.location.posZ = Integer.parseInt( m.group(6) );
    			this.nbt = m.group(7);
    		}
    	} else {
    		CheshireCat.logger.error("Cannot parse BLOCK definition: "+definition);
    	}
	}
	
	@Override
	public boolean build(World world, ChunkCoordinates location, int r) {
		boolean rv;
		ChunkCoordinates blockPos = new ChunkCoordinates();
		
		if( block == null ) {
    		CheshireCat.logger.error("No valid block definition in BLOCK directive");
			return false;
		}
		if( world.isRemote ) {
    		CheshireCat.logger.error("Block placement called with remote world?  Should never happen.");
			return false;
		}
		// get block position, rotated
		blockPos = rotateAndAdd(this.location,r,location);

		// place the block(s), possibly with the NBT in a tileentity
		//CheshireCat.logger.info("Base location  : "+location.posX+","+location.posY+","+location.posZ+" r="+r);
		//CheshireCat.logger.info("Offset         : "+this.location.posX+","+this.location.posY+","+this.location.posZ);
		//CheshireCat.logger.info("Block placement: "+blockPos.posX+","+blockPos.posY+","+blockPos.posZ);
		// 
		BlockPos bp = new BlockPos(blockPos.posX, blockPos.posY, blockPos.posZ);
		IBlockState blockstate = world.getBlockState(bp);
		IBlockState newstate = this.block;
		newstate = CheshireCat.rotateState(newstate,r);
		Block curBlock = blockstate.getBlock();
		int curDv = curBlock.getMetaFromState(blockstate);
		if(this.soft) {
			if( curBlock != Blocks.air && curBlock != Blocks.water && curBlock != Blocks.flowing_water ) {
				return true;  // soft block only placed in air or water
			}
		}
		
		if( curBlock != newstate.getBlock() || curDv != newstate.getBlock().getMetaFromState(newstate) ) {
			rv = world.setBlockState(bp,newstate,2);
			if( !rv ) {
				CheshireCat.logger.warn("Failed to place block in the world");		
				return false;
			}
		
		}
		
		if( this.nbt != null ) {
			try {
				NBTTagCompound nbtTag;
				// read in nbtTag from this.nbt ... very version dependent
				
	//			nbtTag = (NBTTagCompound) JsonToNBT.func_150315_a(this.nbt); // forge 1.7.10
				nbtTag = (NBTTagCompound) JsonToNBT.func_180713_a(this.nbt); // forge 1.8
				nbtTag.setInteger("x", blockPos.posX);
				nbtTag.setInteger("y", blockPos.posY);
				nbtTag.setInteger("z", blockPos.posZ);
				TileEntity te = TileEntity.createAndLoadEntity(nbtTag);
				world.setTileEntity(bp,te);
			} catch( Exception e ) {
				CheshireCat.logger.error("Unable to parse NBT '"+this.nbt+"': "+e.getMessage());		
				return false;
			}
		}
		return true;
	}
	
	@Override
	public String asText() {
		String s = "BLOCK ";
		int thisDv = dv;
		if( this.block == null) {
			return "# BLOCK undefined ";
		}
		if(soft) { 
			s += "~"; 
		}
		s += Block.blockRegistry.getNameForObject(this.block.getBlock());
		// XXXX TODO 
		thisDv = this.block.getBlock().getMetaFromState(this.block);
		if(thisDv>0) {
			s+="("+thisDv+")";
		}
		s += " " + location.posX + "," + location.posY + "," + location.posZ;
		if( nbt != null ) {
			s += " " + nbt;
		}
		return s;
	}
}
